(     Title:  Float Bit Constants
       File:  float-bit-constants.fs
  Test file:  rawfloat-test.fs, ieeefp-ref-test.fs
    Version:  0.9.6
    Revised:  December 22, 2020
     Author:  David N. Williams
    License:  Creative Commons CC0 1.0 Universal

The number of bits per float cannot be found reliably from 1 FLOATS 8 *.
And neither that nor endianness has a standard environmental enquiry.
The brute-force code below uses F-, F0=, and F@ to compare the native
format for -1.0 to the four covered ieeefp formats.

Bit patterns for -1.0:
                             bf800000  single
                    bff00000 00000000  double
               bfff 80000000 00000000  extended double
  bfff0000 00000000 00000000 00000000  quadruple

Version 0.9.6
15Oct20 * Moved from the fMARKER -FLOAT-PARAMS section of rawfloat.fs to
          this file.  Previously logged in the rawfloat part of
          mixfloat.log.  Originated in mixed-float library, July 5,
          2009, or before.
10Dec20 * Removed s" FLOATING-EXT" ENVIRONMENT?  Now fp has to be
          already present.
        * Made conditional definition of QFCREATE stand alone.
)

\ SIGNATURE
\   FLOAT-BIT-CONSTANTS

\ CONSTANTS
\   bits/float  FLITTLE-ENDIAN?  LITTLE-ENDIAN?
\   emax  prec  emin

: FLOAT-BIT-CONSTANTS ;

[UNDEFINED] F@ [IF] cr .( ***Floating point not available.) ABORT [THEN]

MARKER -FLOAT-PARAMS
1 pad !
pad c@ 0= ( bendian?) 0= CONSTANT LITTLE-ENDIAN?

[UNDEFINED] BITS/AU [IF]
s" ADDRESS-UNIT-BITS" ENVIRONMENT? 0=
[IF] cr .( ***Can't determine ADDRESS-UNIT-BITS.) ABORT [THEN]
CONSTANT BITS/AU
[THEN]

[UNDEFINED] BITS/CELL [IF]
bits/au 1 cells * CONSTANT BITS/CELL
[THEN]

[UNDEFINED] QFALIGNED [IF]
: QFALIGNED  ( qfaddr -- qfaddr' )  127 + [ 127 invert ] literal and ;
: QFALIGN    ( -- )                 here qfaligned here - allot ;
[THEN]
[UNDEFINED] QFCREATE [IF]
: QFCREATE  ( "name" -- )  CREATE qfalign DOES> ( -- a-addr) qfaligned ;
[THEN]

\ All of these are 0-padded to 128 bits, trailing the contiguous
\ floating-point data.
bits/cell 32 = [IF]
LITTLE-ENDIAN? [IF]
QFCREATE -1-32le  $bf800000 , 0 , 0 , 0 , 
QFCREATE -1-64le  0 , $bff00000 , 0 , 0 ,
QFCREATE -1-80le  0 , $80000000 , $bfff , 0 ,
QFCREATE -1-128le 0 , 0 , 0 , $bfff0000 , 

[ELSE]  \ BIG-ENDIAN
QFCREATE -1-32be  $bf800000 , 0 , 0 , 0 ,
QFCREATE -1-64be  $bff00000 , 0 , 0 , 0 ,
QFCREATE -1-80be  $bfff , $80000000 , 0 , 0 ,
QFCREATE -1-128be $bfff0000 , 0 , 0 , 0 ,
[THEN]

[ELSE]  \ 64-BITS/CELL
LITTLE-ENDIAN? [IF]
QFCREATE -1-32le  $bf800000 , 0 ,
QFCREATE -1-64le  $bff0000000000000 , 0 ,
QFCREATE -1-80le  $8000000000000000 , $bfff ,
QFCREATE -1-128le 0 , $bfff000000000000 , 

[ELSE]  \ BIG-ENDIAN
QFCREATE -1-32be  $bf800000 , 0 ,
QFCREATE -1-64be  $bff0000000000000 , 0 ,
QFCREATE -1-80be  $bfff , $8000000000000000 ,
QFCREATE -1-128be $bfff000000000000 , 0 ,
[THEN] [THEN]

: -1-fdatum=  ( f: r -- s: flag ) -1e f- f0= ;

0 -2
LITTLE-ENDIAN? [IF]
-1-32le  f@ -1-fdatum= [IF] ( 0) 2drop 32 true  [THEN]
-1-64le  f@ -1-fdatum= [IF] ( 0) 2drop 64 true  [THEN]
-1-80le  f@ -1-fdatum= [IF] ( 0) 2drop 80 true  [THEN]
-1-128le f@ -1-fdatum= [IF] ( 0) 2drop 128 true [THEN]
[ELSE]  \ BIG-ENDIAN 
-1-32be  f@ -1-fdatum= [IF] ( 0) 2drop 32 false [THEN]
-1-64be  f@ -1-fdatum= [IF] ( 0) 2drop 64 false [THEN]
-1-80be  f@ -1-fdatum= [IF] ( 0) 2drop 80 false [THEN]
-1-128be f@ -1-fdatum= [IF] ( 0) 2drop 128 false [THEN]
[THEN]
( {bits/float|0} {fle?|-2})

LITTLE-ENDIAN?  ( {bits/float|0} {fle?|-2} le?)
-FLOAT-PARAMS
CONSTANT FLITTLE-ENDIAN?
CONSTANT LITTLE-ENDIAN?
CONSTANT bits/float

\ These are possible replacements for environmental enquiries in the
\ draft proposal.

bits/float  #32 = [IF]  #24   #127 [THEN] 
bits/float  #64 = [IF]  #53  #1023 [THEN] 
bits/float  #80 = [IF]  #64 #16383 [THEN] 
bits/float #128 = [IF] #113 #16383 [THEN] 
CONSTANT emax  CONSTANT prec   1 emax - CONSTANT emin
