(     Title:  Regression tests for PFE IEEE-754 words
       File:  ieeefp-pfe-test.fs
    Version:  0.6.4
    Revised:  January 3, 2021
     Author:  David N. Williams
    License:  Public Domain

These tests are expected to work on 32- or 64-bit systems for which the
dnw branch of PFE works.

The date above may reflect cosmetic changes not logged here.

Version 0.6.4
31Dec20 * Removed option to use ttester.fs, to avoid s" FLOATING"
          ENVIRONMENT?.
        * Added USE-QUAD-FLOATS switch.
        * Added quad FCONSTANTs.
01Jan21 * Added conditionals for quad status flag tests.
) \     * Added OMIT-(FENABLE) user-config.
(       * Added OMIT-CATCH user-config.
03Jan21 * Updated description of rounding to include quads.

Version 0.6.2
27Aug20 * Revised to use simple hex instead of mixed constants.
29Aug20 * Turned alternate tests back on.  All pass [Catalina] except
          cases where FINEXACT is set simultaneously with FOVERFLOW and
          FUNDERFLOW.
        * Added reports of system values for rounding and exception
          flags.
        * Added report of how cpu resolves the inexact upon overflow or
          underflow system dependence, and made exception tests
          conditional on the outcome.
30Aug20 * Temporarily added TEST-FNEXTAFTER switch for repurposing
          FNEXTUP and FNEXTDOWN tests via redefinition of FNEXTUP and
          FNEXTDOWN in terms of FNEXTAFTER.
        * Catalina tests fail for parenthesized machine language version
          of FENABLE and FDISABLE.
        * Replaced uncommentable alternate exception handler tests by
          tests using CATCH.  Works smoothly.
        * Added regular and alternate tests for FDENORMALOPERAND, when
          it is defined.
02Sep20 * Renamed FEGETROUND and FESETROUND as GET-FROUND and
          SET-FROUND.
        * Rewrote constants using hex-fp interpreted input, with tests.
        * Added >IEEEFLOAT tests.
03Sep20 * Reverted to rawhexfloat.fs conversions for FCONSTANTs.
        * Added hex conversion reproducibility tests, possibly to be
          moved to rawhexfloat.fs.
        * Revised >IEEEFLOAT tests for hex-fp input with embedded white
          space.
        * Added direct tests for interpreted hex-fp.   
04Sep20 * Renamed rounding modes with "FR-" instead of "FE-".
07Seo20 * Added HEX>F reproducibility tests.
08Sep20 * Renamed FDENORMALOPERAND as FDENORMOPAND.
24Dec20 * Replaced loadm ieeefp by loadm floating loadm fpaux.

Version 0.6.0
13Aug20 * Commented out alternative handling for 64 bits.  Works with
          one error on Mojave with 32-bit pfe.
27Aug20 * Put alternative handling back in after 64-bit revisions to
          ieeefp-ext.c  No errors on Mojave 32-bit PFE except the above.
          Individual alternative exceptions seem to work.

Version 0.5.5
02Oct09 * Tests for setting rounding mode state.
        * Tests for exception flag state
03Oct09 * Tests for rounding calculation.
        * Tests for exceptions on calculation.
        * Tests for FNEXTUP and FNEXTDOWN.
        * Tests for alternate exception state.
28Jun10 * Added Krishna Myneni's terminal colors.
        * Tests for alternate exception stopping.
29Jun10 * Revised .EXCEPTS and renamed as .FAE.
        * Added trailing F. to alternate exception stopping
          tests because of alternate exception imprecision with
          linux.
        * Tests for GET-FHANDLER, SET-FHANDLER, INSTALL-FHANDLING.
01Jul10 * Revised alternate stopping commentary and tests to
          distinguish between pre- and post-computation
          exceptions for intel x87.
)

\ *** USER CONFIG

\ Make this TRUE to test with quad floating point.
\
FALSE CONSTANT USE-QUAD-FLOATS

\ Make this TRUE to do limited testing of FNEXAFTER by redefining
\ FNEXTUP AND FNEXTDOWN.
\
TRUE CONSTANT TEST-FNEXTAFTER

\ Make this TRUE to omit (FENABLE) and (FDISABLE) tests.
\
TRUE CONSTANT OMIT-(FENABLE)

\ Make this TRUE to omit CATCH tests which fail with throw code 9 
\ (invalid memory address) on macOS systems with 32-bit cells or 128-bit
\ floats, or a segmentation fault on some linux systems.
\
TRUE CONSTANT OMIT-CATCH

\ *** INCLUDES

USE-QUAD-FLOATS [IF]
LOADM qfloating
INCLUDE qfsynos.fs
[ELSE]
LOADM floating
[THEN]
loadm fpaux

SYNONYM F. FH.  \ for ftester-errors.fs
INCLUDE ttester-xf.fs
INCLUDE tester-display.fs
INCLUDE xtester-errors.fs
INCLUDE ftester-errors.fs
true verbose !
set-exact  \ this is actually the default for ttester-xf.

INCLUDE rawhexfloat.fs
(
This library does conversions among a cell-based raw binary format, a
plain hex string format, and native ieee-fp binary format.  It is used
to define the FCONSTANTs below.

Having discovered that PFE can interpret the ieee hex-fp format, we
tried using that instead, and got it to work after a few stumbles.  The
stumbles were often due to doubts and misunderstandings about how the
gcc implementations of sscanf and stod worked.

We decided to stay with the rawhexfloat library, as it provides more
robust definitions whose only system dependence is in the implementation
of F@ and F!.  It also provides more direct tests of interpreted hex-fp
input.
)
decimal


\ *** REPORTS

?.cr
?." An address unit is " bits/au . ?." bits.
?." There are " bits/cell .  ?." bits/cell.
?." There are " bits/float . ?." bits/float.
LITTLE-ENDIAN?
[IF]   ?." Integer data in memory is little-endian.
[ELSE] ?." Integer data in memory is big-endian.
[THEN]
FLITTLE-ENDIAN?
[IF]   ?." Float data in memory is little-endian.
[ELSE] ?." Float data in memory is big-endian.
[THEN]


\ *** DEBUGGING

[UNDEFINED] \\ [IF]
: \\  ( -- )  -1 parse 2drop BEGIN refill 0= UNTIL ; [THEN]

[UNDEFINED] HEX. [IF]
: hex.  ( u -- )  base @ >r hex . r> base ! ; [THEN]


\ *** FCONSTANTS

0e fabs    FCONSTANT +0
+0 fnegate FCONSTANT -0

bits/float #64 = [IF]
s" 7fef ffff ffff ffff" hex>f FCONSTANT [+max-n]
s" 7fef ffff ffff fffe" hex>f FCONSTANT [+max-n]-down
s" 4000 0000 0000 0001" hex>f FCONSTANT [+2]-up
s" 3fff ffff ffff ffff" hex>f FCONSTANT [+2]-down
s" 3ff0 0000 0000 0001" hex>f FCONSTANT [+1]-up
s" 3fef ffff ffff ffff" hex>f FCONSTANT [+1]-down
s" 0010 0000 0000 0001" hex>f FCONSTANT [+min-n]-up
s" 0010 0000 0000 0000" hex>f FCONSTANT [+min-n]
s" 000f ffff ffff ffff" hex>f FCONSTANT [+max-subn]
s" 0000 0000 0000 0001" hex>f FCONSTANT [+min-subn]

s" 8000 0000 0000 0001" hex>f FCONSTANT [-min-subn]
s" 800f ffff ffff ffff" hex>f FCONSTANT [-max-subn]
s" 8010 0000 0000 0000" hex>f FCONSTANT [-min-n]
s" 8010 0000 0000 0001" hex>f FCONSTANT [-min-+n]-down
s" bfef ffff ffff ffff" hex>f FCONSTANT [-1]-up
s" bff0 0000 0000 0001" hex>f FCONSTANT [-1]-down
s" bfff ffff ffff ffff" hex>f FCONSTANT [-2]-up
s" c000 0000 0000 0001" hex>f FCONSTANT [-2]-down
s" ffef ffff ffff fffe" hex>f FCONSTANT [-max-+n]-up
s" ffef ffff ffff ffff" hex>f FCONSTANT [-max-n]
[THEN]

bits/float #128 = [IF]
s" 7ffe ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [+max-n]
s" 7ffe ffff ffff ffff ffff ffff ffff fffe" hex>f FCONSTANT [+max-n]-down
s" 4000 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [+2]-up
s" 3fff ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [+2]-down
s" 3fff 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [+1]-up
s" 3ffe ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [+1]-down
s" 0001 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [+min-n]-up
s" 0001 0000 0000 0000 0000 0000 0000 0000" hex>f FCONSTANT [+min-n]
s" 0000 ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [+max-subn]
s" 0000 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [+min-subn]

s" 8000 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [-min-subn]
s" 8000 ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [-max-subn]
s" 8001 0000 0000 0000 0000 0000 0000 0000" hex>f FCONSTANT [-min-n]
s" 8001 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [-min-+n]-down
s" bffe ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [-1]-up
s" bfff 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [-1]-down
s" bfff ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [-2]-up
s" c000 0000 0000 0000 0000 0000 0000 0001" hex>f FCONSTANT [-2]-down
s" fffe ffff ffff ffff ffff ffff ffff fffe" hex>f FCONSTANT [-max-+n]-up
s" fffe ffff ffff ffff ffff ffff ffff ffff" hex>f FCONSTANT [-max-n]
[THEN]


\ *** TESTS

TESTING HEX>F reproducibility
\ These tests may get moved to rawhexfloat-test.fs.

hex
bits/cell #64 =  bits/float #64 =  and [IF]
t{ [+max-n]      f>raw -> 7FEFFFFFFFFFFFFF }t
t{ [+max-n]-down f>raw -> 7FEFFFFFFFFFFFFE }t
t{ [+2]-up       f>raw -> 4000000000000001 }t
t{ [+2]-down     f>raw -> 3FFFFFFFFFFFFFFF }t
t{ [+1]-up       f>raw -> 3FF0000000000001 }t
t{ [+1]-down     f>raw -> 3FEFFFFFFFFFFFFF }t
t{ [+min-n]-up   f>raw -> 0010000000000001 }t
t{ [+min-n]      f>raw -> 0010000000000000 }t
t{ [+max-subn]   f>raw -> 000FFFFFFFFFFFFF }t
t{ [+min-subn]   f>raw -> 0000000000000001 }t

t{ [-min-subn]    f>raw -> 8000000000000001 }t
t{ [-max-subn]    f>raw -> 800FFFFFFFFFFFFF }t
t{ [-min-n]       f>raw -> 8010000000000000 }t
t{ [-min-+n]-down f>raw -> 8010000000000001 }t
t{ [-1]-up        f>raw -> BFEFFFFFFFFFFFFF }t
t{ [-1]-down      f>raw -> BFF0000000000001 }t
t{ [-2]-up        f>raw -> BFFFFFFFFFFFFFFF }t
t{ [-2]-down      f>raw -> C000000000000001 }t
t{ [-max-+n]-up   f>raw -> FFEFFFFFFFFFFFFE }t
t{ [-max-n]       f>raw -> FFEFFFFFFFFFFFFF }t
[THEN]

bits/cell #64 =  bits/float #128 =  and [IF]
t{ [+max-n]      f>raw -> ffffffffffffffff 7ffeffffffffffff }t
t{ [+max-n]-down f>raw -> fffffffffffffffe 7ffeffffffffffff }t
t{ [+2]-up       f>raw -> 0000000000000001 4000000000000000 }t
t{ [+2]-down     f>raw -> ffffffffffffffff 3fffffffffffffff }t
t{ [+1]-up       f>raw -> 0000000000000001 3fff000000000000 }t
t{ [+1]-down     f>raw -> ffffffffffffffff 3ffeffffffffffff }t
t{ [+min-n]-up   f>raw -> 0000000000000001 0001000000000000 }t
t{ [+min-n]      f>raw -> 0000000000000000 0001000000000000 }t
t{ [+max-subn]   f>raw -> ffffffffffffffff 0000ffffffffffff }t
t{ [+min-subn]   f>raw -> 0000000000000001 0000000000000000 }t

t{ [-min-subn]    f>raw -> 0000000000000001 8000000000000000 }t
t{ [-max-subn]    f>raw -> ffffffffffffffff 8000ffffffffffff }t
t{ [-min-n]       f>raw -> 0000000000000000 8001000000000000 }t
t{ [-min-+n]-down f>raw -> 0000000000000001 8001000000000000 }t
t{ [-1]-up        f>raw -> ffffffffffffffff bffeffffffffffff }t
t{ [-1]-down      f>raw -> 0000000000000001 bfff000000000000 }t
t{ [-2]-up        f>raw -> ffffffffffffffff bfffffffffffffff }t
t{ [-2]-down      f>raw -> 0000000000000001 c000000000000000 }t
t{ [-max-+n]-up   f>raw -> fffffffffffffffe fffeffffffffffff }t
t{ [-max-n]       f>raw -> ffffffffffffffff fffeffffffffffff }t
[THEN]

bits/cell #32 =  bits/float #64 =  and [IF]
t{ [+max-n]      f>raw ->  FFFFFFFF 7FEFFFFF }t
t{ [+max-n]-down f>raw ->  FFFFFFFE 7FEFFFFF }t
t{ [+2]-up       f>raw ->  00000001 40000000 }t
t{ [+2]-down     f>raw ->  FFFFFFFF 3FFFFFFF }t
t{ [+1]-up       f>raw ->  00000001 3FF00000 }t
t{ [+1]-down     f>raw ->  FFFFFFFF 3FEFFFFF }t
t{ [+min-n]-up   f>raw ->  00000001 00100000 }t
t{ [+min-n]      f>raw ->  00000000 00100000 }t
t{ [+max-subn]   f>raw ->  FFFFFFFF 000FFFFF }t
t{ [+min-subn]   f>raw ->  00000001 00000000 }t

t{ [-min-subn]    f>raw -> 00000001 80000000 }t
t{ [-max-subn]    f>raw -> FFFFFFFF 800FFFFF }t
t{ [-min-n]       f>raw -> 00000000 80100000 }t
t{ [-min-+n]-down f>raw -> 00000001 80100000 }t
t{ [-1]-up        f>raw -> FFFFFFFF BFEFFFFF }t
t{ [-1]-down      f>raw -> 00000001 BFF00000 }t
t{ [-2]-up        f>raw -> FFFFFFFF BFFFFFFF }t
t{ [-2]-down      f>raw -> 00000001 C0000000 }t
t{ [-max-+n]-up   f>raw -> FFFFFFFE FFEFFFFF }t
t{ [-max-n]       f>raw -> FFFFFFFF FFEFFFFF }t
[THEN]

bits/cell #32 =  bits/float #128 =  and [IF]
t{ [+max-n]      f>raw -> ffffffff ffffffff ffffffff 7ffeffff }t
t{ [+max-n]-down f>raw -> fffffffe ffffffff ffffffff 7ffeffff }t
t{ [+2]-up       f>raw -> 00000001 00000000 00000000 40000000 }t
t{ [+2]-down     f>raw -> ffffffff ffffffff ffffffff 3fffffff }t
t{ [+1]-up       f>raw -> 00000001 00000000 00000000 3fff0000}t
t{ [+1]-down     f>raw -> ffffffff ffffffff ffffffff 3ffeffff }t
t{ [+min-n]-up   f>raw -> 00000001 00000000 00000000 00010000 }t
t{ [+min-n]      f>raw -> 00000000 00000000 00000000 00010000 }t
t{ [+max-subn]   f>raw -> ffffffff ffffffff ffffffff 0000ffff }t
t{ [+min-subn]   f>raw -> 00000001 00000000 00000000 00000000 }t

t{ [-min-subn]    f>raw -> 00000001 00000000 00000000 80000000 }t
t{ [-max-subn]    f>raw -> ffffffff ffffffff ffffffff 8000ffff }t
t{ [-min-n]       f>raw -> 00000000 00000000 00000000 80010000 }t
t{ [-min-+n]-down f>raw -> 00000001 00000000 00000000 80010000 }t
t{ [-1]-up        f>raw -> ffffffff ffffffff ffffffff bffeffff }t
t{ [-1]-down      f>raw -> 00000001 00000000 00000000 bfff0000 }t
t{ [-2]-up        f>raw -> ffffffff ffffffff ffffffff bfffffff }t
t{ [-2]-down      f>raw -> 00000001 00000000 00000000 c0000000 }t
t{ [-max-+n]-up   f>raw -> fffffffe ffffffff ffffffff fffeffff }t
t{ [-max-n]       f>raw -> ffffffff ffffffff ffffffff fffeffff }t
[THEN]


TESTING >IEEEFLOAT with hex-fp input
decimal

: white?  ( char -- flag )  33 u< ;     \ Tool Belt 2002 Is-White

: -ws  ( s -- s' )
(
Remove all white space from the ANS Forth string s and leave the
compacted string s'.  The new string body address is at PAD.  This works
even if the input string s starts somewhere in PAD, but then it may get
partially written over.
)
  pad >r
  BEGIN
    ( len) dup 0>
  WHILE
    over c@ white? 0=
    IF over c@ r@ c! r> 1+ >r THEN
    1 /string
  REPEAT 2drop
  pad r> pad - ;

: -ws>ieeefloat  ( s -- f: r )
  -ws >ieeefloat 0= ABORT" >IEEEFLOAT: Can't convert." ;

bits/float #64 = [IF]
t{ s"  0x1.F FFFF FFFF FFFF p+1023" -ws>ieeefloat -> [+max-n] }t
t{ s"  0x1.F FFFF FFFF FFFE p+1023" -ws>ieeefloat -> [+max-n]-down }t
t{ s"  0x1.0 0000 0000 0001 p1"     -ws>ieeefloat -> [+2]-up }t
t{ s"  0x1.F FFFF FFFF FFFF p0"     -ws>ieeefloat -> [+2]-down }t
t{ s"  0x1.0 0000 0000 0001 p0"     -ws>ieeefloat -> [+1]-up }t
t{ s"  0x1.F FFFF FFFF FFFF p-1"    -ws>ieeefloat -> [+1]-down }t
t{ s"  0x1.0 0000 0000 0001 p-1022" -ws>ieeefloat -> [+min-n]-up }t
t{ s"  0x1.0 0000 0000 0000 p-1022" -ws>ieeefloat -> [+min-n]  }t
t{ s"  0x0.F FFFF FFFF FFFF p-1022" -ws>ieeefloat -> [+max-subn] }t
t{ s"  0x0.0 0000 0000 0001 p-1022" -ws>ieeefloat -> [+min-subn] }t

t{ s" -0x0.0 0000 0000 0001 p-1022" -ws>ieeefloat -> [-min-subn] }t
t{ s" -0x0.F FFFF FFFF FFFF p-1022" -ws>ieeefloat -> [-max-subn] }t
t{ s" -0x1.0 0000 0000 0000 p-1022" -ws>ieeefloat -> [-min-n]  }t
t{ s" -0x1.0 0000 0000 0001 p-1022" -ws>ieeefloat -> [-min-+n]-down }t
t{ s" -0x1.F FFFF FFFF FFFF p-1"    -ws>ieeefloat -> [-1]-up }t
t{ s" -0x1.0 0000 0000 0001 p0"     -ws>ieeefloat -> [-1]-down }t
t{ s" -0x1.F FFFF FFFF FFFF p0"     -ws>ieeefloat -> [-2]-up }t
t{ s" -0x1.0 0000 0000 0001 p1"     -ws>ieeefloat -> [-2]-down }t
t{ s" -0x1.F FFFF FFFF FFFE p+1023" -ws>ieeefloat -> [-max-+n]-up }t
t{ s" -0x1.F FFFF FFFF FFFF p+1023" -ws>ieeefloat -> [-max-n] }t
[THEN]

bits/float #128 = [IF]
t{ s"  0x1.ffff ffff ffff ffff ffff ffff ffff p+16383" -ws>ieeefloat -> [+max-n] }t 
t{ s"  0x1.ffff ffff ffff ffff ffff ffff fffe p+16383" -ws>ieeefloat -> [+max-n]-down }t
t{ s"  0x1.0000 0000 0000 0000 0000 0000 0001 p1"      -ws>ieeefloat -> [+2]-up }t
t{ s"  0x1.ffff ffff ffff ffff ffff ffff ffff p0"      -ws>ieeefloat -> [+2]-down }t
t{ s"  0x1.0000 0000 0000 0000 0000 0000 0001 p0"      -ws>ieeefloat -> [+1]-up }t
t{ s"  0x1.ffff ffff ffff ffff ffff ffff ffff p-1"     -ws>ieeefloat -> [+1]-down }t
t{ s"  0x1.0000 0000 0000 0000 0000 0000 0001 p-16382" -ws>ieeefloat -> [+min-n]-up }t
t{ s"  0x1.0000 0000 0000 0000 0000 0000 0000 p-16382" -ws>ieeefloat -> [+min-n]  }t
t{ s"  0x0.ffff ffff ffff ffff ffff ffff ffff p-16382" -ws>ieeefloat -> [+max-subn] }t
t{ s"  0x0.0000 0000 0000 0000 0000 0000 0001 p-16382" -ws>ieeefloat -> [+min-subn] }t

t{ s" -0x0.0000 0000 0000 0000 0000 0000 0001 p-16382" -ws>ieeefloat -> [-min-subn] }t
t{ s" -0x0.ffff ffff ffff ffff ffff ffff ffff p-16382" -ws>ieeefloat -> [-max-subn] }t
t{ s" -0x1.0000 0000 0000 0000 0000 0000 0000 p-16382" -ws>ieeefloat -> [-min-n]  }t
t{ s" -0x1.0000 0000 0000 0000 0000 0000 0001 p-16382" -ws>ieeefloat -> [-min-+n]-down }t
t{ s" -0x1.ffff ffff ffff ffff ffff ffff ffff p-1"     -ws>ieeefloat -> [-1]-up }t
t{ s" -0x1.0000 0000 0000 0000 0000 0000 0001 p0"      -ws>ieeefloat -> [-1]-down }t
t{ s" -0x1.ffff ffff ffff ffff ffff ffff ffff p0"      -ws>ieeefloat -> [-2]-up }t
t{ s" -0x1.0000 0000 0000 0000 0000 0000 0001 p1"      -ws>ieeefloat -> [-2]-down }t
t{ s" -0x1.ffff ffff ffff ffff ffff ffff fffe p+16383" -ws>ieeefloat -> [-max-+n]-up }t
t{ s" -0x1.ffff ffff ffff ffff ffff ffff ffff p+16383" -ws>ieeefloat -> [-max-n] }t
[THEN]

TESTING interpreted hex-fp
decimal

bits/float #64 = [IF]
t{  0x1.fffffffffffffp+1023 -> [+max-n] }t
t{  0x1.ffffffffffffep+1023 -> [+max-n]-down }t
t{  0x1.0000000000001p+1    -> [+2]-up }t
t{  0x1.fffffffffffffp+0    -> [+2]-down }t
t{  0x1.0000000000001p+0    -> [+1]-up }t
t{  0x1.fffffffffffffp-1    -> [+1]-down }t
t{  0x1.0000000000001p-1022 -> [+min-n]-up }t
t{  0x1.0000000000000p-1022 -> [+min-n]  }t
t{  0x0.fffffffffffffp-1022 -> [+max-subn] }t
t{  0x0.0000000000001p-1022 -> [+min-subn] }t

t{ -0x0.0000000000001p-1022 -> [-min-subn] }t
t{ -0x0.fffffffffffffp-1022 -> [-max-subn] }t
t{ -0x1.0000000000000p-1022 -> [-min-n] }t
t{ -0x1.0000000000001p-1022 -> [-min-+n]-down }t
t{ -0x1.fffffffffffffp-1    -> [-1]-up }t
t{ -0x1.0000000000001p+0    -> [-1]-down }t
t{ -0x1.fffffffffffffp+0    -> [-2]-up }t
t{ -0x1.0000000000001p+1    -> [-2]-down }t
t{ -0x1.ffffffffffffep+1023 -> [-max-+n]-up }t
t{ -0x1.fffffffffffffp+1023 -> [-max-n] }t
[THEN]

bits/float #128 = [IF]
t{  0x1.ffffffffffffffffffffffffffffp+16383 -> [+max-n] }t
t{  0x1.fffffffffffffffffffffffffffep+16383 -> [+max-n]-down }t
t{  0x1.0000000000000000000000000001p+1     -> [+2]-up }t
t{  0x1.ffffffffffffffffffffffffffffp+0     -> [+2]-down }t
t{  0x1.0000000000000000000000000001p+0     -> [+1]-up }t
t{  0x1.ffffffffffffffffffffffffffffp-1     -> [+1]-down }t
t{  0x1.0000000000000000000000000001p-16382 -> [+min-n]-up }t
t{  0x1.0000000000000000000000000000p-16382 -> [+min-n]  }t
t{  0x0.ffffffffffffffffffffffffffffp-16382 -> [+max-subn] }t
t{  0x0.0000000000000000000000000001p-16382 -> [+min-subn] }t

t{ -0x0.0000000000000000000000000001p-16382 -> [-min-subn] }t
t{ -0x0.ffffffffffffffffffffffffffffp-16382 -> [-max-subn] }t
t{ -0x1.0000000000000000000000000000p-16382 -> [-min-n] }t
t{ -0x1.0000000000000000000000000001p-16382 -> [-min-+n]-down }t
t{ -0x1.ffffffffffffffffffffffffffffp-1     -> [-1]-up }t
t{ -0x1.0000000000000000000000000001p+0     -> [-1]-down }t
t{ -0x1.ffffffffffffffffffffffffffffp+0     -> [-2]-up }t
t{ -0x1.0000000000000000000000000001p+1     -> [-2]-down }t
t{ -0x1.fffffffffffffffffffffffffffep+16383 -> [-max-+n]-up }t
t{ -0x1.ffffffffffffffffffffffffffffp+16383 -> [-max-n] }t
[THEN]

bits/float #64 = [IF]
0x1.921fb54442d18p+1 FCONSTANT pi \ pi correctly rounded
\ Fractional approximation to pi provided by Albert van der Horst, c.l.f.
t{ 1570796326794896619e 500000000000000000e f/ pi f- -> 0e }t
[THEN]

TEST-FNEXTAFTER [IF]
: FNEXTUP   +inf fnextafter ;
: FNEXTDOWN -inf fnextafter ;
TESTING +inf FNEXTAFTER
[ELSE]
TESTING FNEXTUP
[THEN]

\ nan propagation
t{ +nan FNEXTUP -> +nan }t
t{ -nan FNEXTUP -> -nan }t

\ [+max-n] boundary
t{ +inf          FNEXTUP -> +inf }t
t{ [+max-n]      FNEXTUP -> +inf }t
t{ [+max-n]-down FNEXTUP -> [+max-n] }t

\ +normal integer boundaries
t{  2e       FNEXTUP -> [+2]-up }t
t{ [+2]-down FNEXTUP ->  2e }t
t{  1e       FNEXTUP -> [+1]-up }t
t{ [+1]-down FNEXTUP ->  1e }t

\ +subnormal boundaries
t{ [+min-subn] FNEXTUP -> [+min-subn] f2* }t
t{ [+max-subn] FNEXTUP -> [+min-n] }t
t{ [+min-n]    FNEXTUP -> [+min-n]-up }t

\ +/-0 boundaries
t{ +0          FNEXTUP -> [+min-subn] }t
t{ -0          FNEXTUP -> [+min-subn] }t
t{ [-min-subn] FNEXTUP -> -0 }t

\ -subnormal boundaries
t{ [-min-subn] f2* FNEXTUP -> [-min-subn] }t
t{ [-min-+n]-down  FNEXTUP -> [-min-n] }t
t{ [-min-n]        FNEXTUP -> [-max-subn] }t

\ -normal integer boundaries
t{ -1e       FNEXTUP -> [-1]-up }t
t{ [-1]-down FNEXTUP -> -1e }t
t{ -2e       FNEXTUP -> [-2]-up }t 
t{ [-2]-down FNEXTUP -> -2e }t

\ [-max-n] boundary
t{ [-max-n] FNEXTUP -> [-max-+n]-up }t
t{ -inf     FNEXTUP -> [-max-n] }t

TEST-FNEXTAFTER [IF]
TESTING -inf FNEXTAFTER
[ELSE]
TESTING FNEXTDOWN
[THEN]

\ nan propagation
t{ +nan FNEXTDOWN -> +nan }t
t{ -nan FNEXTDOWN -> -nan }t

\ [+max-n] boundary
t{ +inf     FNEXTDOWN -> [+max-n] }t
t{ [+max-n] FNEXTDOWN -> [+max-n]-down }t

\ +normal integer boundaries
t{ [+2]-up FNEXTDOWN ->  2e }t
t{  2e     FNEXTDOWN -> [+2]-down }t
t{ [+1]-up FNEXTDOWN ->  1e }t
t{  1e     FNEXTDOWN -> [+1]-down }t

\ +subnormal boundaries
t{ [+min-n]-up     FNEXTDOWN -> [+min-n] }t
t{ [+min-n]        FNEXTDOWN -> [+max-subn] }t 
t{ [+min-subn] f2* FNEXTDOWN -> [+min-subn] }t 

\ +/-0 boundaries
t{ [+min-subn] FNEXTDOWN -> +0 }t
t{ +0          FNEXTDOWN -> [-min-subn] }t
t{ -0          FNEXTDOWN -> [-min-subn] }t

\ -subnormal boundaries
t{ [-min-subn] FNEXTDOWN -> [-min-subn] f2* }t
t{ [-max-subn] FNEXTDOWN -> [-min-n] }t
t{ [-min-n]    FNEXTDOWN -> [-min-+n]-down }t

\ -normal integer boundaries
t{ [-1]-up FNEXTDOWN -> -1e }t 
t{ -1e     FNEXTDOWN -> [-1]-down }t 
t{ [-2]-up FNEXTDOWN -> -2e }t 
t{ -2e     FNEXTDOWN -> [-2]-down }t 

\ [-max-n] boundary
t{ [-max-+n]-up FNEXTDOWN -> [-max-n] }t
t{ [-max-n]     FNEXTDOWN -> -inf }t
t{ -inf         FNEXTDOWN -> -inf }t

TESTING SET-FROUND GET-FROUND

: hexu.r   ( u n -- ) base @ >r hex u.r r> base ! ;
: ?hexu.r  ( u -- )   VERBOSE @ IF hexu.r ELSE 2drop THEN ;

?."   FR-DOWNWARD = " FR-DOWNWARD   4 ?hexu.r ?.cr
?."     FR-UPWARD = " FR-UPWARD     4 ?hexu.r ?.cr
?." FR-TOWARDZERO = " FR-TOWARDZERO 4 ?hexu.r ?.cr       
?."  FR-TONEAREST = " FR-TONEAREST  4 ?hexu.r ?.cr   

t{ get-fround -> FR-TONEAREST }t  \ default
t{ FR-DOWNWARD   set-fround get-fround -> FR-DOWNWARD }t
t{ FR-UPWARD     set-fround get-fround -> FR-UPWARD }t
t{ FR-TOWARDZERO set-fround get-fround -> FR-TOWARDZERO }t
t{ FR-TONEAREST  set-fround get-fround -> FR-TONEAREST }t

(
The hex-digit double calculations below are enough to distinguish all
the rounding cases.  "Rounds" means the exact result has more than 53
bits of significance.  "Exact" means the exact result has exactly 53
significant bits.

The layout below is for doubles, but as indicated by the example layout
for quads, the ideas is exactly the same for them.  Inspection of the
last hex digit is key for understanding when rounding occurs.

A criterion for the tests is that at least one result shall be different
for each rounding mode, and of course that all results shall be correct.
The roundTiesToEven mode has the richest set of possibilities, and
determines how many tests are needed.

  1.0 0000 0000 0000      #1       1.0000 0000 0000 0000 0000 0000 0000
+ 0.0 0000 0000 0000 8    eps    + 0.0000 0000 0000 0000 0000 0000 0000 8
----------------------            ----------------------------------------  
  1.0 0000 0000 0000 8    rounds   1.0000 0000 0000 0000 0000 0000 0000 8

  1.0 0000 0000 0001      #2
+ 0.0 0000 0000 0000 8    eps
----------------------
  1.0 0000 0000 0001 8    rounds

  0.F FFFF FFFF FFFF 8    #3
+ 0.0 0000 0000 0000 4    eps/2
----------------------
  0.F FFFF FFFF FFFF C    rounds


  1.0 0000 0000 0000      #4
- 0.0 0000 0000 0000 8    eps
----------------------
  0.F FFFF FFFF FFFF 8    exact =[+1]-down

  1.0 0000 0000 0001      #5
- 0.0 0000 0000 0000 8    eps
----------------------
  1.0 0000 0000 0000 8    rounds

  0.F FFFF FFFF FFFF 8    #6
- 0.0 0000 0000 0000 4    eps/2
----------------------
  0.F FFFF FFFF FFFF 4    rounds


 -1.0 0000 0000 0000      #7
+ 0.0 0000 0000 0000 8    eps
----------------------
 -0.F FFFF FFFF FFFF 8    exact = [-1]-up

 -1.0 0000 0000 0001      #8
+ 0.0 0000 0000 0000 8    eps
----------------------
 -1.0 0000 0000 0000 8    rounds

 -0.F FFFF FFFF FFFF 8    #9
+ 0.0 0000 0000 0000 4    eps/2
----------------------
 -0.F FFFF FFFF FFFF 4    rounds


 -1.0 0000 0000 0000      #10
- 0.0 0000 0000 0000 8    eps
----------------------
 -1.0 0000 0000 0000 8    rounds

 -1.0 0000 0000 0001      #11
- 0.0 0000 0000 0000 8    eps
----------------------
 -1.0 0000 0000 0001 8    rounds

 -0.F FFFF FFFF FFFF 8    #12
- 0.0 0000 0000 0000 4    eps/2
----------------------
 -0.F FFFF FFFF FFFF C    rounds
)

0 [IF]
1e fnextup 1e f- f2/ FCONSTANT 2^-53 
2^-53 f2/            FCONSTANT 2^-54
[THEn]
1e fnextup 1e f- f2/ FCONSTANT eps 
eps f2/              FCONSTANT eps/2

FR-TONEAREST set-fround
t{  1e       eps   f+ ->  1e }t              \ #1
t{ [+1]-up   eps   f+ -> [+1]-up fnextup }t  \ #2
t{ [+1]-down eps/2 f+ -> 1e }t               \ #3

t{  1e       eps   f- -> [+1]-down }t            \ #4
t{ [+1]-up   eps   f- ->  1e }t                  \ #5
t{ [+1]-down eps/2 f- -> [+1]-down fnextdown }t  \ #6

t{ -1e       eps   f+ -> [-1]-up }t          \ #7
t{ [-1]-down eps   f+ -> -1e }t              \ #8
t{ [-1]-up   eps/2 f+ -> [-1]-up fnextup }t  \ #9

t{ -1e       eps   f- -> -1e }t                  \ #10
t{ [-1]-down eps   f- -> [-1]-down fnextdown }t  \ #11
t{ [-1]-up   eps/2 f- -> -1e }t                  \ #12

FR-TOWARDZERO set-fround
t{  1e       eps   f+ ->  1e }t        \ #1
t{ [+1]-up   eps   f+ -> [+1]-up }t    \ #2
t{ [+1]-down eps/2 f+ -> [+1]-down }t  \ #3

t{  1e       eps   f- -> [+1]-down }t            \ #4
t{ [+1]-up   eps   f- -> 1e }t                   \ #5
t{ [+1]-down eps/2 f- -> [+1]-down fnextdown }t  \ #6

t{ -1e       eps   f+ -> [-1]-up }t          \ #7
t{ [-1]-down eps   f+ -> -1e }t              \ #8
t{ [-1]-up   eps/2 f+ -> [-1]-up fnextup }t  \ #9

t{ -1e       eps   f- -> -1e }t        \ #10
t{ [-1]-down eps   f- -> [-1]-down }t  \ #11
t{ [-1]-up   eps/2 f- -> [-1]-up }t    \ #12

FR-UPWARD set-fround
t{  1e       eps   f+ -> [+1]-up }t          \ #1
t{ [+1]-up   eps   f+ -> [+1]-up fnextup }t  \ #2
t{ [+1]-down eps/2 f+ -> 1e }t               \ #3

t{  1e       eps   f- -> [+1]-down }t  \ #4
t{ [+1]-up   eps   f- -> [+1]-up }t    \ #5
t{ [+1]-down eps/2 f- -> [+1]-down }t  \ #6

t{ -1e       eps   f+ -> [-1]-up }t          \ #7
t{ [-1]-down eps   f+ -> -1e }t              \ #8
t{ [-1]-up   eps/2 f+ -> [-1]-up fnextup }t  \ #9

t{ -1e       eps   f- -> -1e }t        \ #10
t{ [-1]-down eps   f- -> [-1]-down }t  \ #11
t{ [-1]-up   eps/2 f- -> [-1]-up }t    \ #12

FR-DOWNWARD set-fround
t{  1e       eps   f+ ->  1e }t        \ #1
t{ [+1]-up   eps   f+ -> [+1]-up }t    \ #2
t{ [+1]-down eps/2 f+ -> [+1]-down }t  \ #3

t{  1e       eps   f- -> [+1]-down }t            \ #4
t{ [+1]-up   eps   f- ->  1e }t                  \ #5
t{ [+1]-down eps/2 f- -> [+1]-down fnextdown }t  \ #6

t{ -1e       eps   f+ -> [-1]-up }t    \ #7
t{ [-1]-down eps   f+ -> [-1]-down }t  \ #8
t{ [-1]-up   eps/2 f+ -> [-1]-up }t    \ #9

t{ -1e       eps   f- -> [-1]-down }t            \ #10
t{ [-1]-down eps   f- -> [-1]-down fnextdown }t  \ #11
t{ [-1]-up   eps/2 f- -> -1e }t                  \ #12

FR-TONEAREST set-fround  \ restore default

TESTING SET-FFLAGS GET-FFLAGS CLEAR-FFLAGS

?."    FDIVBYZERO = " FDIVBYZERO 4 ?hexu.r ?.cr
?."      FINEXACT = " FINEXACT   4 ?hexu.r ?.cr
?."      FINVALID = " FINVALID   4 ?hexu.r ?.cr      
?."     FOVERFLOW = " FOVERFLOW  4 ?hexu.r ?.cr  
?."    FUNDERFLOW = " FUNDERFLOW 4 ?hexu.r ?.cr 
[DEFINED] FDENORMOPAND [IF]
?."  FDENORMOPAND = " FDENORMOPAND 4 ?hexu.r ?.cr 
[THEN]
?."  ALL-FEXCEPTS = " ALL-FEXCEPTS 4 ?hexu.r ?.cr 
(
FDENORMOPAND is explained in the following quote from macOS
Catalina:
/Library/Developer/CommandLineTools/SDKs/MacOSX.sdk/usr/include/fenv.h
----------
#define FE_INEXACT          0x0020
#define FE_UNDERFLOW        0x0010
#define FE_OVERFLOW         0x0008
#define FE_DIVBYZERO        0x0004
#define FE_INVALID          0x0001
/*  FE_DENORMALOPERAND
    An Intel-specific flag that is raised when an operand to a floating-point
    arithmetic operation is denormal, or a single- or double-precision denormal
    value is loaded on the x87 stack.  This flag is not raised by SSE
    arithmetic when the DAZ control bit is set.                               */
#define FE_DENORMALOPERAND  0x0002
#define FE_ALL_EXCEPT       0x003f
----------
It is not implemented in Linux Mint 19.
)

: allfl@  ( -- excepts ) ALL-FEXCEPTS get-fflags ;
: 0allfl  ( -- )         ALL-FEXCEPTS clear-fflags ;

\ This is used for passing an accumulation of flags on to a later test.
VARIABLE lastfls

t{ 0allfl allfl@ -> 0 }t

(
It is implementation-dependent whether the inexact fp flag is also set
when the overflow or underflow fp flags are set.  The following reports
the implementation for the host cpu.
)  

0allfl FOVERFLOW set-fflags   allfl@ FOVERFLOW FINEXACT or =
[IF]
?." FINEXACT is raised when FOVERFLOW is set.
[ELSE]
?." FINEXACT is not raised when FOVERFLOW is set.
[THEN]

0allfl FUNDERFLOW set-fflags   allfl@ FUNDERFLOW FINEXACT or =
[IF]
?." FINEXACT is raised when FUNDERFLOW is set.
[ELSE]
?." FINEXACT is not raised when FUNDERFLOW is set.
[THEN]

t{ 0allfl FDIVBYZERO set-fflags allfl@ -> FDIVBYZERO }t
t{ FOVERFLOW FINEXACT or set-fflags allfl@
     -> FDIVBYZERO FOVERFLOW or FINEXACT or }t

t{ FINEXACT set-fflags   allfl@ -> FDIVBYZERO FOVERFLOW or FINEXACT or }t
t{ FINEXACT clear-fflags allfl@ -> FDIVBYZERO FOVERFLOW or }t

t{ FUNDERFLOW set-fflags allfl@ FINEXACT or
     -> FDIVBYZERO FOVERFLOW or FUNDERFLOW or FINEXACT or }t

t{ FINEXACT clear-fflags FINVALID set-fflags allfl@
     -> FDIVBYZERO FOVERFLOW or FUNDERFLOW or FINVALID or dup lastfls ! }t
t{ FDIVBYZERO clear-fflags allfl@
     -> lastfls @ FDIVBYZERO xor dup lastfls ! }t
t{ FOVERFLOW clear-fflags allfl@
     -> lastfls @ FOVERFLOW xor }t

t{ FUNDERFLOW FINEXACT or FINVALID or clear-fflags allfl@
     -> 0 }t

t{ 0allfl 3e 0e f/ fdrop allfl@ -> FDIVBYZERO }t
t{ 0allfl 0e 0e f/ fdrop allfl@ -> FINVALID }t

\ For Linux Mint 19, FINEXACT is raised on overflow here, even though
\ merely setting FOVERFLOW does not do so.
bits/float #64 = [IF]
t{ 0allfl 1.5E+308 1E-3 f/ fdrop allfl@ FINEXACT or -> FOVERFLOW FINEXACT or }t
[ELSE]  \ bits/float #128 =
t{ 0allfl 1.1E+4932 1E-3 f/ fdrop allfl@ FINEXACT or -> FOVERFLOW FINEXACT or }t
[THEN]

\ Neither Catalina nor Linux Mint 19 raises FINEXACT here.  Exact
\ rounding?
bits/float #64 = [IF]
t{ 0allfl 1E-3 1.5E+308 f/ fdrop allfl@ -> FUNDERFLOW FINEXACT or }t
[ELSE]  \ bits/float #128 =
t{ 0allfl 1E-3 1.1E+4932 f/ fdrop allfl@ -> FUNDERFLOW FINEXACT or }t
[THEN]

t{ 0allfl 1e eps/2 f+ fdrop allfl@ -> FINEXACT }t
t{ 0allfl 1e eps   f+ allfl@       -> 1e FINEXACT }t \ exact rounding

[DEFINED] FDENORMOPAND [IF]
t{ 0allfl 0e [+min-subn] f+ FDENORMOPAND get-fflags fdrop
     -> FDENORMOPAND }t \ [+min-subn]
t{ 0allfl [+max-subn] 0e f+ FDENORMOPAND get-fflags fdrop
     -> FDENORMOPAND }t \ [+max-subn]
[THEN]
0allfl

TESTING FENABLE FDISABLE FENABLED INSTALL-FHANDLING SET-FHANDLER GET-FHANDLER

: 0allfae  ( -- )  ALL-FEXCEPTS fdisable ;

: my-fhandler  ( throw -- ) cr ." MY-FHANDLER: throw = " dup . throw ;

\ These tests check that loading fpaux properly initializes (installs)
\ fhandling to the simple THROW handler, and reinitializes fhandling after
\ the handler has been changed.
\
t{ get-fhandler -> ' throw }t
t{ ' my-fhandler set-fhandler -> }t
t{ get-fhandler -> ' my-fhandler }t
t{ install-fhandling -> }t
t{ get-fhandler -> ' throw }t

t{ fenabled -> 0 }t  \ system default
t{ 0allfae  -> }t
t{ fenabled -> 0 }t

t{ FDIVBYZERO fenable fenabled -> FDIVBYZERO }t
t{ FINVALID   fenable fenabled -> FDIVBYZERO FINVALID or dup lastfls ! }t
t{ FOVERFLOW FUNDERFLOW or FINEXACT or fenable fenabled
     -> lastfls @ FOVERFLOW or FUNDERFLOW or FINEXACT or dup lastfls ! }t
[DEFINED] FDENORMOPAND [IF]
t{ FDENORMOPAND fenable fenabled -> ALL-FEXCEPTS }t
[THEN]

t{ FUNDERFLOW fdisable fenabled -> ALL-FEXCEPTS FUNDERFLOW invert and }t
t{ 0allfae fenabled -> 0 }t

OMIT-(FENABLE) [IF]

\ These words do direct control register access and are superfluous.
?." Omitting tests for machine words (FENABLE) and (FDISABLE).

[ELSE]

\ Machine words:  (fenable)  (fdisable)

: (0allfae)  ( -- )  ALL-FEXCEPTS (fdisable) ;

t{ fenabled -> 0 }t  \ system default
t{ (0allfae)  -> }t
t{ fenabled -> 0 }t

t{ FDIVBYZERO (fenable) fenabled -> FDIVBYZERO }t
t{ FINVALID   (fenable) fenabled -> FDIVBYZERO FINVALID or }t
t{ FOVERFLOW FUNDERFLOW FINEXACT or or (fenable) fenabled -> ALL-FEXCEPTS }t

t{ FUNDERFLOW (fdisable) fenabled -> ALL-FEXCEPTS FUNDERFLOW invert and }t
t{ (0allfae) fenabled -> 0 }t

[THEN]  \ OMIT-(FENABLE)

OMIT-CATCH [IF]

?." Omitting CATCH tests which fail with throw code -9,
?."   invalid memory address.

[ELSE]

' my-fhandler set-fhandler
\ throw        SIGFPE        Forth 2012 description
  -42 CONSTANT FPE_FLTDIV  \ floating-point divide by zero
  -46 CONSTANT FPE_FLTINV  \ floating-point invalid argument
  -43 CONSTANT FPE_FLTOVF  \ floating-point result out of range
  -54 CONSTANT FPE_FLTUND  \ floating-point underflow
  -41 CONSTANT FPE_FLTRES  \ loss of precision
  -55 CONSTANT FPE_FLTUID  \ floating-point unidentified fault [temporary]

0allfae
t{ FDIVBYZERO fenable 1e 0e ' f/ CATCH fdrop fdrop -> FPE_FLTDIV }t
t{ FINVALID   fenable 0e 0e ' f/ CATCH fdrop fdrop -> FPE_FLTINV }t
bits/float #64 = [IF]
t{ FOVERFLOW  fenable 1.5E+308 1E-3 ' f/ CATCH fdrop fdrop -> FPE_FLTOVF }t
t{ FUNDERFLOW fenable 1E-3 1.5E+308 ' f/ CATCH fdrop fdrop -> FPE_FLTUND }t
[ELSE]  \ bits/flot #128
t{ FOVERFLOW  fenable 1.1E+4932 1E-3 ' f/ CATCH fdrop fdrop -> FPE_FLTOVF }t
t{ FUNDERFLOW fenable 1E-3 1.1E+4932 ' f/ CATCH fdrop fdrop -> FPE_FLTUND }t
[THEN]
t{ FINEXACT   fenable 1e eps/2 ' f+ CATCH fdrop fdrop -> FPE_FLTRES }t
[DEFINED] FDENORMOPAND [IF]
t{ FDENORMOPAND fenable 0e [+min-subn] ' f- CATCH fdrop fdrop -> FPE_FLTUID }t
[THEN]
0allfae

[THEN]  \ OMIT-CATCH

(
The following tests and explanation are for reference only.  They are
for double floats only, and are obsoleted by the above CATCH tests.

Uncomment one of the following to test an alternate, stopping exception.
The trailing F. on the last three below caters to the intel Linux we
use, whose gcc defaults to the x87 instead of the SIMD fpu.  With intel,
the last three are post-computation exceptions, which do not trigger
until the next fp operation, F. in this case.  The first two are
precomputation exceptions, which trigger at the operation.  Furthermore,
alternate exceptions are imprecise for x87.  For example, substituting
an fp number for F. produces a segmentation fault in our Linux tests,
without calling the fhandler.

The exceptions are precise with the ppc and intel SIMD fpu's. SIMD is
the default for Snow Leopard.  The trailing F. is superfluous for
testing with Leopard ppc and Snow Leopard intel.
)
\ FDIVBYZERO fenable 1e 0e f/
\ FINVALID fenable 0e 0e f/
\ FOVERFLOW fenable 1.5E+308 1E-3 f/ f.
\ FUNDERFLOW fenable 1E-3 1.5E+308 f/ f.
\ FINEXACT fenable 1e eps/2 f+ f.

?.cr
blue-text ?.xt-errors s"     " ?type ?.ft-errors normal-text
?.cr
